// dspPlugin.h
#ifndef DSPPLUGIN_H
#define DSPPLUGIN_H
#include <windows.h>
#include <cstdarg>
#include <vector>
// header version: 0x20 == 0.20 == winamp 2.0
const int DSP_HDRVER = 0x20;
// this structure, defined by Winamp, must not change in format
struct winampDSPModule {
	char* description;		// description
	HWND hwndParent;		// filled in by calling app
	HINSTANCE hDllInstance;	// filled in by calling app
	void (*Config)(winampDSPModule* this_mod);
	int (*Init)(winampDSPModule* this_mod);     // 0 on success
	// modify waveform samples
	// returns number of samples to write
	// must return <= numsamples*2 && >= max(128,numsamples/2) 
	int (*ModifySamples)(winampDSPModule* this_mod, void* samples, 
		int numsamples, int bps, int nch, int srate);
	void (*Quit)(winampDSPModule* this_mod);
	void* userData;			// user data, optional
};
// this structure, defined by Winamp, must not change in format
struct winampDSPHeader {
	int version;       // DSP_HDRVER
	const char* description; // description of library
	winampDSPModule* (*getModule)(int);	// module retrieval function
};
// DSPPlugin class encapsulates the Winamp DSP plug-in API
class DSPPlugin	{
	vector<winampDSPModule*> modules;
public:
	static DSPPlugin* plugin;
	static winampDSPModule* getModule(int which);
	winampDSPHeader hdr;
	DSPPlugin(const char* desc,...);
};
#endif // DSPPLUGIN_H
