using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Text;
using System.Reflection;
using GDepends.Analyzer;

namespace GDepends
{
    /// <summary>
    /// UI for the GDEPENDS application
    /// </summary>
    public class MainForm : System.Windows.Forms.Form
    {
        private System.Windows.Forms.TreeView tvItems;
        private System.Windows.Forms.Button btnOpen;
        private System.Windows.Forms.Button btnGraph;
        private System.Windows.Forms.Button btnExit;
        private System.Windows.Forms.OpenFileDialog dlgFileOpen;
        private System.Windows.Forms.Button btnHide;
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.Container components = null;

        public MainForm()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();
        }

        protected override void Dispose( bool disposing )
        {
            if( disposing )
            {
                if (components != null) 
                {
                    components.Dispose();
                }
            }
            base.Dispose( disposing );
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.tvItems = new System.Windows.Forms.TreeView();
            this.btnExit = new System.Windows.Forms.Button();
            this.btnGraph = new System.Windows.Forms.Button();
            this.btnOpen = new System.Windows.Forms.Button();
            this.dlgFileOpen = new System.Windows.Forms.OpenFileDialog();
            this.btnHide = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // tvItems
            // 
            this.tvItems.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
                | System.Windows.Forms.AnchorStyles.Left) 
                | System.Windows.Forms.AnchorStyles.Right);
            this.tvItems.ImageIndex = -1;
            this.tvItems.Name = "tvItems";
            this.tvItems.SelectedImageIndex = -1;
            this.tvItems.Size = new System.Drawing.Size(264, 304);
            this.tvItems.TabIndex = 0;
            this.tvItems.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.tvItems_AfterSelect);
            // 
            // btnExit
            // 
            this.btnExit.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
            this.btnExit.Location = new System.Drawing.Point(288, 143);
            this.btnExit.Name = "btnExit";
            this.btnExit.TabIndex = 3;
            this.btnExit.Text = "E&xit";
            this.btnExit.Click += new System.EventHandler(this.btnExit_Click);
            // 
            // btnGraph
            // 
            this.btnGraph.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
            this.btnGraph.Enabled = false;
            this.btnGraph.Location = new System.Drawing.Point(288, 53);
            this.btnGraph.Name = "btnGraph";
            this.btnGraph.TabIndex = 2;
            this.btnGraph.Text = "&Graph";
            this.btnGraph.Click += new System.EventHandler(this.btnGraph_Click);
            // 
            // btnOpen
            // 
            this.btnOpen.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
            this.btnOpen.Location = new System.Drawing.Point(288, 8);
            this.btnOpen.Name = "btnOpen";
            this.btnOpen.TabIndex = 1;
            this.btnOpen.Text = "&Open";
            this.btnOpen.Click += new System.EventHandler(this.btnOpen_Click);
            // 
            // dlgFileOpen
            // 
            this.dlgFileOpen.RestoreDirectory = true;
            // 
            // btnHide
            // 
            this.btnHide.Enabled = false;
            this.btnHide.Location = new System.Drawing.Point(288, 98);
            this.btnHide.Name = "btnHide";
            this.btnHide.TabIndex = 4;
            this.btnHide.Text = "&Hide";
            this.btnHide.Click += new System.EventHandler(this.btnHide_Click);
            // 
            // MainForm
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(376, 301);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.btnHide,
                                                                          this.btnExit,
                                                                          this.btnGraph,
                                                                          this.btnOpen,
                                                                          this.tvItems});
            this.Name = "MainForm";
            this.Text = "GDepends";
            this.Load += new System.EventHandler(this.MainForm_Load);
            this.ResumeLayout(false);

        }
        #endregion

        [STAThread]
        static void Main() 
        {
            Application.Run(new MainForm());
        }

        private void MainForm_Load(object sender, System.EventArgs e)
        {
            LoadPlugins();
        }

        private void btnGraph_Click(object sender, System.EventArgs e)
        {
            CreateGraph();
        }

        private void btnOpen_Click(object sender, System.EventArgs e)
        {

            OpenFile();
        }

        private void btnExit_Click(object sender, System.EventArgs e)
        {
            Close();
        }

        private void btnHide_Click(object sender, System.EventArgs e)
        {
            HideSelectedNode();
        }

        private void tvItems_AfterSelect(object sender, 
                            System.Windows.Forms.TreeViewEventArgs e)
        {
            btnHide.Enabled = true;
        }

        /// <summary>
        ///  Reads the xml config file and load the requested plugins.
        ///  The plugins are registered with the AnalyzerFactory to 
        ///  retrieve the file types available for processing, and the
        ///  file types are loaded into the Filter property of the 
        ///  file open dialog
        /// </summary>
        private void LoadPlugins()
        {
            Plugins plugins = new Plugins();
            plugins.ReadXml(PLUGINS_FILENAME);

            ArrayList pluginAttributes = new ArrayList();

            for(int i = 0; i < plugins.Assembly.Count; i++)
            {
                Assembly assembly;
                assembly = Assembly.LoadFrom(plugins.Assembly[i].Name);
                AnalyzerFactory.RegisterAssembly(assembly, 
                                                 pluginAttributes);
            }

            StringBuilder filter = new StringBuilder();
            foreach(PluginAttribute attribute in pluginAttributes)
            {
                // after the first iteration add a | seperator
                if(filter.Length > 0)
                {
                    filter.Append("|");
                }
                filter.Append(String.Format("{0}|*.{1}", 
                                            attribute.Description, 
                                            attribute.FileExtension));
            }

            dlgFileOpen.Filter = filter.ToString();
        }
    
        /// <summary>
        /// Create a graph of dependencies
        /// </summary>
        private void CreateGraph()
        {
            if(analyzer != null)
            {
                analyzer.Display();
            }
        }

        /// <summary>
        /// Query the user for the file to open and load the 
        /// appropiate analyzer. The file is then parsed and 
        /// placed into a tree view
        /// </summary>
        private void OpenFile()
        {
            dlgFileOpen.FileName = null;
            if(dlgFileOpen.ShowDialog() == DialogResult.OK)
            {
                string extension;
                FileInfo fileInfo = new FileInfo(dlgFileOpen.FileName);

                // grab the file extension and strip the leading "."
                extension = fileInfo.Extension;
                extension = extension.Substring(1);
                
                analyzer = AnalyzerFactory.CreateAnalyzer(extension);
                analyzer.Parse(new StreamReader(dlgFileOpen.FileName));

                PopulateTreeView(analyzer.Dependencies);
                btnGraph.Enabled = true;
            }
        }

        /// <summary>
        /// Mark the currently selected node as hidden and 
        /// refresh the display
        /// </summary>
        private void HideSelectedNode()
        {
            TreeNode treeNode = tvItems.SelectedNode;
            if(treeNode != null)
            {
                foreach(DictionaryEntry d in analyzer.Dependencies)
                {
                    DependencyNode depNode = (DependencyNode)d.Value;
                    if(depNode.Name.CompareTo(treeNode.Text) == 0)
                    { 
                        depNode.Visible = false;
                        PopulateTreeView(analyzer.Dependencies);
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// Clear the current tree view and readd the nodes
        /// </summary>
        /// <param name="dependencies">A collection of 
        ///                            DependencyNodes</param>
        private void PopulateTreeView(Hashtable dependencies)
        {
            tvItems.Nodes.Clear();

            foreach(DictionaryEntry d in dependencies)
            {
                DependencyNode depNode = (DependencyNode)d.Value;
                if(depNode.Visible)
                {
                    AddNode(tvItems.Nodes, depNode);
                }
            }

        }

        /// <summary>
        /// Add the specified node to the tree and recursively 
        /// add all the child nodes
        /// </summary>
        /// <param name="treeNodes">A tree node collection</param>
        /// <param name="depNode">A DependencyNode</param>
        private void AddNode(TreeNodeCollection treeNodes, 
                             DependencyNode depNode)
        {
            TreeNode treeNode = new TreeNode(depNode.Name);
            
            // add to the top level of nodes
            treeNodes.Add(treeNode);

            foreach(DependencyNode child in depNode.Dependencies)
            {
                if(child.Visible)
                {
                    // add to the current tree node
                    AddNode(treeNode.Nodes, child);
                }
            }       
        }

        private DependencyAnalyzer analyzer;
        private const string PLUGINS_FILENAME = "Plugins.xml";
    }
}
